#' @import lme4
#' @importFrom MASS ginv

#' @importFrom utils head
#' @importFrom stats coef anova
#' 
#' @importFrom parallel clusterCall clusterExport clusterSetRNGStream
#'     mclapply detectCores makeCluster stopCluster
#'
#' @importClassesFrom Matrix Matrix
#' @importFrom Matrix Matrix sparseMatrix rankMatrix
#' @importMethodsFrom Matrix t isSymmetric "%*%" solve diag chol
#'     chol2inv forceSymmetric "*"
#'
#' @importFrom graphics abline legend lines plot
#' @importFrom methods as is
#' @importFrom stats as.formula family formula getCall logLik
#'     model.matrix pchisq pf pgamma printCoefmat quantile simulate
#'     terms update update.formula var vcov sigma
#'
#' @importFrom broom tidy
#' @export tidy
#' 
#' @importFrom dplyr as_tibble


#' @title pbkrtest internal
#' @description pbkrtest internal
#' @name internal-pbkrtest
#'
NULL

