% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SAT_modcomp.R
\name{compute_auxiliary}
\alias{compute_auxiliary}
\title{Compute_auxiliary quantities needed for the Satterthwaite
approximation.}
\usage{
compute_auxiliary(model, tol = 1e-06)
}
\arguments{
\item{model}{A linear mixed model object}

\item{tol}{A tolerance}
}
\value{
A list
}
\description{
Computes variance-covariance matrix of variance parameters (theta, sigma), the Jacobian of
each variance parameter etc.
}
\author{
Søren Højsgaard
}
\keyword{internal}
