% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal_pbkrtest.R
\name{internal}
\alias{internal}
\alias{print.PBmodcomp}
\alias{print.summary_PBmodcomp}
\alias{summary.PBmodcomp}
\alias{plot.PBmodcomp}
\alias{summary.KRmodcomp}
\alias{print.KRmodcomp}
\alias{KRmodcomp_init}
\alias{KRmodcomp_init.lmerMod}
\alias{KRmodcomp_init.mer}
\alias{as.data.frame.XXmodcomp}
\alias{tidy}
\title{Internal functions for the pbkrtest package}
\description{
These functions are not intended to be called directly.
}
